@@ -3,6 +3,8 @@
 from __future__ import annotations
 
 import ast
+import collections.abc
+import enum
 import importlib
 import inspect
 import re
@@ -14,20 +16,25 @@ from typing import TYPE_CHECKING, Any, AnyStr, ForwardRef, NewType, TypeVar, Uni
 
 from docutils import nodes
 from docutils.frontend import get_default_settings
-from sphinx.ext.autodoc.mock import mock  # type: ignore[attr-defined]
+from docutils.parsers.rst import Directive, directives
+from docutils.utils import new_document
+from sphinx.ext.autodoc.mock import mock
 from sphinx.parsers import RSTParser
 from sphinx.util import logging, rst
+from sphinx.util.docutils import sphinx_domains
 from sphinx.util.inspect import TypeAliasForwardRef, stringify_signature
 from sphinx.util.inspect import signature as sphinx_signature
 
-from ._parser import parse
+from ._parser import _RstSnippetParser, parse
 from .patches import install_patches
 from .version import __version__
 
 if TYPE_CHECKING:
+    import optparse
     from ast import FunctionDef, Module, stmt
     from collections.abc import Callable
 
+    from docutils.frontend import Values
     from docutils.nodes import Node
     from docutils.parsers.rst import states
     from sphinx.application import Sphinx
@@ -36,6 +43,8 @@ if TYPE_CHECKING:
     from sphinx.ext.autodoc import Options
 
 _LOGGER = logging.getLogger(__name__)
+
+_BUILTIN_DIRECTIVES = frozenset(directives._directive_registry)  # noqa: SLF001
 _PYDATA_ANNOTS_TYPING = {
     "Any",
     "AnyStr",
@@ -72,7 +81,7 @@ _TYPES_DICT[types.FunctionType] = "FunctionType"
 
 
 class MyTypeAliasForwardRef(TypeAliasForwardRef):
-    def __or__(self, value: Any) -> Any:
+    def __or__(self, value: Any) -> Any:  # ty: ignore[invalid-method-override]
         return Union[self, value]  # noqa: UP007
 
 
@@ -103,9 +112,9 @@ def get_annotation_module(annotation: Any) -> str:
     ):
         return "typing"
     if hasattr(annotation, "__module__"):
-        return annotation.__module__  # type: ignore[no-any-return]
+        return annotation.__module__
     if hasattr(annotation, "__origin__"):
-        return annotation.__origin__.__module__  # type: ignore[no-any-return]
+        return annotation.__origin__.__module__
     msg = f"Cannot determine the module of {annotation}"
     raise ValueError(msg)
 
@@ -134,19 +143,19 @@ def get_annotation_class_name(annotation: Any, module: str) -> str:  # noqa: C90
         return "NewType"
 
     if getattr(annotation, "__qualname__", None):
-        return annotation.__qualname__  # type: ignore[no-any-return]
+        return annotation.__qualname__
     if getattr(annotation, "_name", None):  # Required for generic aliases on Python 3.7+
-        return annotation._name  # type: ignore[no-any-return]  # noqa: SLF001
+        return annotation._name  # noqa: SLF001
     if module in {"typing", "typing_extensions"} and isinstance(getattr(annotation, "name", None), str):
         # Required for at least Pattern and Match
-        return annotation.name  # type: ignore[no-any-return]
+        return annotation.name
 
     origin = getattr(annotation, "__origin__", None)
     if origin:
         if getattr(origin, "__qualname__", None):  # Required for Protocol subclasses
-            return origin.__qualname__  # type: ignore[no-any-return]
+            return origin.__qualname__
         if getattr(origin, "_name", None):  # Required for Union on Python 3.7+
-            return origin._name  # type: ignore[no-any-return]  # noqa: SLF001
+            return origin._name  # noqa: SLF001
 
     annotation_cls = annotation if inspect.isclass(annotation) else type(annotation)
     return annotation_cls.__qualname__.lstrip("_")
@@ -175,13 +184,13 @@ def get_annotation_args(annotation: Any, module: str, class_name: str) -> tuple[
     if class_name == "ClassVar" and hasattr(annotation, "__type__"):  # ClassVar on Python < 3.7
         return (annotation.__type__,)
     if class_name == "TypeVar" and hasattr(annotation, "__constraints__"):
-        return annotation.__constraints__  # type: ignore[no-any-return]
+        return annotation.__constraints__
     if class_name == "NewType" and hasattr(annotation, "__supertype__"):
         return (annotation.__supertype__,)
     if class_name == "Literal" and hasattr(annotation, "__values__"):
-        return annotation.__values__  # type: ignore[no-any-return]
+        return annotation.__values__
     if class_name == "Generic":
-        return annotation.__parameters__  # type: ignore[no-any-return]
+        return annotation.__parameters__
     result = getattr(annotation, "__args__", ())
     # 3.10 and earlier Tuple[()] returns ((), ) instead of () the tuple does
     return () if len(result) == 1 and result[0] == () else result  # type: ignore[misc]
@@ -211,6 +220,17 @@ def fixup_module_name(config: Config, module: str) -> str:
     return module
 
 
+def _format_literal_arg(arg: Any, config: Config) -> str:
+    if isinstance(arg, enum.Enum):
+        enum_cls = type(arg)
+        module = fixup_module_name(config, enum_cls.__module__)
+        fully_qualified = getattr(config, "typehints_fully_qualified", False)
+        qualified = f"{module}.{enum_cls.__qualname__}.{arg.name}" if module else f"{enum_cls.__qualname__}.{arg.name}"
+        prefix = "" if fully_qualified or not module else "~"
+        return f":py:attr:`{prefix}{qualified}`"
+    return f"``{arg!r}``"
+
+
 def format_annotation(annotation: Any, config: Config, *, short_literals: bool = False) -> str:  # noqa: C901, PLR0911, PLR0912, PLR0915, PLR0914
     """
     Format the annotation.
@@ -266,12 +286,16 @@ def format_annotation(annotation: Any, config: Config, *, short_literals: bool =
 
     # Some types require special handling
     if full_name == "typing.NewType":
-        args_format = f"\\(``{annotation.__name__}``, {{}})"
-        role = "obj"
-    elif full_name == "typing.Annotated":
+        newtype_module = fixup_module_name(config, getattr(annotation, "__module__", ""))
+        newtype_name = annotation.__name__
+        newtype_qualified = f"{newtype_module}.{newtype_name}" if newtype_module else newtype_name
+        newtype_prefix = "" if fully_qualified or not newtype_module else "~"
+        supertype = format_annotation(annotation.__supertype__, config, short_literals=short_literals)
+        return f":py:class:`{newtype_prefix}{newtype_qualified}` ({supertype})"
+    if full_name == "typing.Annotated":
         # By default we don't show metadata in Annotated
         return format_annotation(annotation.__origin__, config, short_literals=short_literals)
-    elif full_name in {"typing.TypeVar", "typing.ParamSpec"}:
+    if full_name in {"typing.TypeVar", "typing.ParamSpec"}:
         params = {k: getattr(annotation, f"__{k}__") for k in ("bound", "covariant", "contravariant")}
         params = {k: v for k, v in params.items() if v}
         if "bound" in params:
@@ -299,9 +323,10 @@ def format_annotation(annotation: Any, config: Config, *, short_literals: bool =
         fmt = [format_annotation(arg, config, short_literals=short_literals) for arg in args]
         formatted_args = f"\\[\\[{', '.join(fmt[:-1])}], {fmt[-1]}]"
     elif full_name == "typing.Literal":
+        literal_parts = [_format_literal_arg(arg, config) for arg in args]
         if short_literals:
-            return f"\\{' | '.join(f'``{arg!r}``' for arg in args)}"
-        formatted_args = f"\\[{', '.join(f'``{arg!r}``' for arg in args)}]"
+            return f"\\{' | '.join(literal_parts)}"
+        formatted_args = f"\\[{', '.join(literal_parts)}]"
     elif is_bars_union:
         if not args:
             return f":py:{'class' if sys.version_info >= (3, 14) else 'data'}:`{prefix}typing.Union`"
@@ -394,7 +419,7 @@ def process_signature(  # noqa: C901, PLR0911, PLR0912, PLR0913, PLR0917
         return None
 
     try:
-        obj = inspect.unwrap(obj)
+        obj = inspect.unwrap(obj)  # ty: ignore[invalid-argument-type]
     except ValueError:
         return None
     sph_signature = sphinx_signature(obj, type_aliases=app.config["autodoc_type_aliases"])
@@ -404,7 +429,7 @@ def process_signature(  # noqa: C901, PLR0911, PLR0912, PLR0913, PLR0917
         if typehints_formatter is None:
             return annotation
         formatted_name = typehints_formatter(annotation)
-        return annotation if not isinstance(formatted_name, str) else TypeVar(formatted_name)
+        return annotation if not isinstance(formatted_name, str) else TypeVar(formatted_name)  # ty: ignore[invalid-legacy-type-variable]
 
     if app.config.typehints_use_signature_return:
         sph_signature = sph_signature.replace(
@@ -444,7 +469,7 @@ def process_signature(  # noqa: C901, PLR0911, PLR0912, PLR0913, PLR0917
             if method_name.startswith("__") and not method_name.endswith("__"):
                 # when method starts with double underscore Python applies mangling -> prepend the class name
                 method_name = f"_{obj.__qualname__.split('.')[-2]}{method_name}"
-            method_object = outer.__dict__[method_name] if outer else obj
+            method_object = outer.__dict__.get(method_name, obj) if outer else obj
             if not isinstance(method_object, classmethod | staticmethod):
                 start = 1
 
@@ -515,24 +540,29 @@ def _execute_guarded_code(autodoc_mock_imports: list[str], obj: Any, module_code
     for _, part in _TYPE_GUARD_IMPORT_RE.findall(module_code):
         guarded_code = textwrap.dedent(part)
         try:
-            try:
-                with mock(autodoc_mock_imports):
-                    exec(guarded_code, getattr(obj, "__globals__", obj.__dict__))  # noqa: S102
-            except ImportError as exc:
-                # ImportError might have occurred because the module has guarded code as well,
-                # so we recurse on the module.
-                if exc.name:
-                    _resolve_type_guarded_imports(autodoc_mock_imports, importlib.import_module(exc.name))
-
-                    # Retry the guarded code and see if it works now after resolving all nested type guards.
-                    with mock(autodoc_mock_imports):
-                        exec(guarded_code, getattr(obj, "__globals__", obj.__dict__))  # noqa: S102
+            _run_guarded_import(autodoc_mock_imports, obj, guarded_code)
         except Exception as exc:  # noqa: BLE001
             _LOGGER.warning(
                 "Failed guarded type import with %r", exc, type="sphinx_autodoc_typehints", subtype="guarded_import"
             )
 
 
+def _run_guarded_import(autodoc_mock_imports: list[str], obj: Any, guarded_code: str) -> None:
+    ns = getattr(obj, "__globals__", obj.__dict__)
+    try:
+        with mock(autodoc_mock_imports):
+            exec(guarded_code, ns)  # noqa: S102
+    except ImportError as exc:
+        if not exc.name:
+            return
+        _resolve_type_guarded_imports(autodoc_mock_imports, importlib.import_module(exc.name))
+        try:
+            with mock(autodoc_mock_imports):
+                exec(guarded_code, ns)  # noqa: S102
+        except ImportError:
+            pass  # third-party internal import that doesn't exist at runtime
+
+
 def _resolve_type_guarded_imports(autodoc_mock_imports: list[str], obj: Any) -> None:
     if _should_skip_guarded_import_resolution(obj):
         return
@@ -552,10 +582,29 @@ def _resolve_type_guarded_imports(autodoc_mock_imports: list[str], obj: Any) ->
             _execute_guarded_code(autodoc_mock_imports, obj, module_code)
 
 
+def _add_type_params_to_localns(
+    obj: Any, localns: dict[Any, MyTypeAliasForwardRef]
+) -> dict[Any, MyTypeAliasForwardRef]:
+    if type_params := getattr(obj, "__type_params__", None):
+        localns = {**localns, **{p.__name__: p for p in type_params}}
+    qualname = getattr(obj, "__qualname__", "") or ""
+    parts = qualname.rsplit(".", 1)
+    if len(parts) > 1:
+        parent_name = parts[0]
+        ns = getattr(obj, "__globals__", None)
+        if ns is None:
+            module = inspect.getmodule(obj)
+            ns = vars(module) if module else None
+        if ns and (parent := ns.get(parent_name)) and (parent_params := getattr(parent, "__type_params__", None)):
+            localns = {**localns, **{p.__name__: p for p in parent_params}}
+    return localns
+
+
 def _get_type_hint(
     autodoc_mock_imports: list[str], name: str, obj: Any, localns: dict[Any, MyTypeAliasForwardRef]
 ) -> dict[str, Any]:
     _resolve_type_guarded_imports(autodoc_mock_imports, obj)
+    localns = _add_type_params_to_localns(obj, localns)
     try:
         result = get_type_hints(obj, None, localns, include_extras=True)
     except (AttributeError, TypeError, RecursionError) as exc:
@@ -914,50 +963,23 @@ class InsertIndexInfo:
 PARAM_SYNONYMS = ("param ", "parameter ", "arg ", "argument ", "keyword ", "kwarg ", "kwparam ")
 
 
-def node_line_no(node: Node) -> int | None:
-    """
-    Get the 1-indexed line on which the node starts if possible. If not, return None.
-
-    Descend through the first children until we locate one with a line number or return None if None of them have one.
-
-    I'm not aware of any rst on which this returns None, to find out would require a more detailed analysis of the
-    docutils rst parser source code. An example where the node doesn't have a line number but the first child does is
-    all `definition_list` nodes. It seems like bullet_list and option_list get line numbers, but enum_list also doesn't.
-    """
-    if node is None:
-        return None
-
-    while node.line is None and node.children:
-        node = node.children[0]
-    return node.line
-
-
-def tag_name(node: Node) -> str:
-    return node.tagname  # type:ignore[attr-defined,no-any-return]
-
-
 def get_insert_index(app: Sphinx, lines: list[str]) -> InsertIndexInfo | None:
     # 1. If there is an existing :rtype: anywhere, don't insert anything.
     if any(line.startswith(":rtype:") for line in lines):
         return None
 
-    # 2. If there is a :returns: anywhere, either modify that line or insert
-    #    just before it.
+    # 2. If there is a :returns: anywhere, either modify that line or insert just before it.
     for at, line in enumerate(lines):
         if line.startswith((":return:", ":returns:")):
             return InsertIndexInfo(insert_index=at, found_return=True)
 
     # 3. Insert after the parameters.
-    # To find the parameters, parse as a docutils tree.
     settings = get_default_settings(RSTParser)  # type: ignore[arg-type]
     settings.env = app.env
-    doc = parse("\n".join(lines), settings)
+    doc = _safe_parse("\n".join(lines), settings)
 
-    # Find a top level child which is a field_list that contains a field whose
-    # name starts with one of the PARAM_SYNONYMS. This is the parameter list. We
-    # hope there is at most of these.
     for child in doc.children:
-        if tag_name(child) != "field_list":
+        if _tag_name(child) != "field_list":
             continue
 
         if not any(c.children[0].astext().startswith(PARAM_SYNONYMS) for c in child.children):
@@ -968,17 +990,17 @@ def get_insert_index(app: Sphinx, lines: list[str]) -> InsertIndexInfo | None:
         # If there is a next sibling but we can't locate a line number, insert
         # at end. (I don't know of any input where this happens.)
         next_sibling = child.next_node(descend=False, siblings=True)
-        line_no = node_line_no(next_sibling) if next_sibling else None
+        line_no = _node_line_no(next_sibling) if next_sibling else None
         at = max(line_no - 2, 0) if line_no else len(lines)
         return InsertIndexInfo(insert_index=at, found_param=True)
 
     # 4. Insert before examples
     for child in doc.children:
-        if tag_name(child) in {"literal_block", "paragraph", "field_list"}:
+        if _tag_name(child) in {"literal_block", "paragraph", "field_list"}:
             continue
-        line_no = node_line_no(child)
+        line_no = _node_line_no(child)
         at = max(line_no - 2, 0) if line_no else len(lines)
-        if lines[at - 1]:  # skip if something on this line
+        if lines[at - 1]:
             break
         return InsertIndexInfo(insert_index=at, found_directive=True)
 
@@ -986,7 +1008,59 @@ def get_insert_index(app: Sphinx, lines: list[str]) -> InsertIndexInfo | None:
     return InsertIndexInfo(insert_index=len(lines))
 
 
-def _inject_rtype(  # noqa: C901, PLR0913, PLR0917
+def _safe_parse(inputstr: str, settings: Values | optparse.Values) -> nodes.document:
+    """
+    Parse RST without triggering extension directive side-effects.
+
+    Replaces non-builtin directive lookups with a no-op handler during parsing
+    to prevent duplicate ID registration and other side-effects from third-party
+    extensions like sphinx-needs.
+    """
+    original_lookup = directives.directive
+
+    def _safe_directive_lookup(
+        directive_name: str,
+        language_module: Any,
+        document: Any,
+    ) -> tuple[type[Directive] | None, list[Any]]:
+        cls, messages = original_lookup(directive_name, language_module, document)
+        if cls is not None and directive_name not in _BUILTIN_DIRECTIVES:
+            return _NoOpDirective, messages
+        return cls, messages
+
+    doc = new_document("", settings=settings)  # ty: ignore[invalid-argument-type]
+    with sphinx_domains(settings.env):
+        directives.directive = _safe_directive_lookup  # type: ignore[assignment]
+        try:
+            parser = _RstSnippetParser()
+            parser.parse(inputstr, doc)
+        finally:
+            directives.directive = original_lookup
+    return doc
+
+
+class _NoOpDirective(Directive):
+    has_content = True
+    optional_arguments = 99
+    final_argument_whitespace = True
+
+    def run(self) -> list[nodes.Node]:  # noqa: PLR6301
+        return []
+
+
+def _node_line_no(node: Node) -> int | None:
+    if node is None:
+        return None
+    while node.line is None and node.children:
+        node = node.children[0]
+    return node.line
+
+
+def _tag_name(node: Node) -> str:
+    return node.tagname
+
+
+def _inject_rtype(  # noqa: C901, PLR0911, PLR0913, PLR0917
     type_hints: dict[str, Any],
     original_obj: Any,
     app: Sphinx,
@@ -1002,6 +1076,8 @@ def _inject_rtype(  # noqa: C901, PLR0913, PLR0917
         return
     if not app.config.typehints_document_rtype_none and type_hints["return"] is types.NoneType:
         return
+    if _has_yields_section(lines) and _is_generator_type(type_hints["return"]):
+        return
 
     r = get_insert_index(app, lines)
     if r is None:
@@ -1020,10 +1096,11 @@ def _inject_rtype(  # noqa: C901, PLR0913, PLR0917
     if r.found_param and insert_index < len(lines) and lines[insert_index].strip():
         insert_index -= 1
 
-    if insert_index == len(lines) and not r.found_param:
+    if insert_index > 0 and insert_index <= len(lines) and lines[insert_index - 1].strip():
         # ensure that :rtype: doesn't get joined with a paragraph of text
-        lines.append("")
+        lines.insert(insert_index, "")
         insert_index += 1
+
     if app.config.typehints_use_rtype or not r.found_return:
         line = f":rtype: {formatted_annotation}"
         lines.insert(insert_index, line)
@@ -1034,6 +1111,23 @@ def _inject_rtype(  # noqa: C901, PLR0913, PLR0917
         lines[insert_index] = f":return: {formatted_annotation} --{line[line.find(' ') :]}"
 
 
+_GENERATOR_TYPES = frozenset({
+    collections.abc.Generator,
+    collections.abc.Iterator,
+    collections.abc.AsyncGenerator,
+    collections.abc.AsyncIterator,
+})
+
+
+def _is_generator_type(annotation: Any) -> bool:
+    origin = getattr(annotation, "__origin__", None)
+    return origin in _GENERATOR_TYPES or annotation in _GENERATOR_TYPES
+
+
+def _has_yields_section(lines: list[str]) -> bool:
+    return any(line.lstrip().startswith((":Yields:", ":yields:", ":yield:")) for line in lines)
+
+
 def validate_config(app: Sphinx, env: BuildEnvironment, docnames: list[str]) -> None:  # noqa: ARG001
     valid = {None, "comma", "braces", "braces-after"}
     if app.config.typehints_defaults not in valid | {False}:
